﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 프로세스 헬퍼
    /// </summary>
    public class ProcessHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 작업 객체 생성하기 - CreateJobObject(securityAttribute, name)

        /// <summary>
        /// 작업 객체 생성하기
        /// </summary>
        /// <param name="securityAttribute">보안 어트리뷰트</param>
        /// <param name="name">명칭</param>
        /// <returns>작업 객체 핸들</returns>
        [DllImport("kernel32.dll", EntryPoint = "CreateJobObjectW", CharSet = CharSet.Unicode)]
        private static extern IntPtr CreateJobObject(SecurityAttribute securityAttribute, string name);

        #endregion
        #region 프로세스를 작업 객체에 할당하기 - AssignProcessToJobObject(jobHandle, processHandle)

        /// <summary>
        /// 프로세스를 작업 객체에 할당하기
        /// </summary>
        /// <param name="jobHandle">작업 객체 핸들</param>
        /// <param name="processHandle">프로세스 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("kernel32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool AssignProcessToJobObject(IntPtr jobHandle, IntPtr processHandle);

        #endregion
        #region 정보 작업 객체 설정하기 - SetInformationJobObject(jobHandle, jobObjectInformationClass, jobObjectInformationHandle, jobObjectInformationLength)

        /// <summary>
        /// 정보 작업 객체 설정하기
        /// </summary>
        /// <param name="jobHandle">작업 핸들</param>
        /// <param name="jobObjectInformationClass">작업 객체 정보 클래스</param>
        /// <param name="jobObjectInformationHandle">작업 객체 정보 핸들</param>
        /// <param name="jobObjectInformationLength">작업 객체 정보 길이</param>
        /// <returns>처리 결과</returns>
        [DllImport("kernel32.dll")]
        private static extern bool SetInformationJobObject(IntPtr jobHandle, JobObjectInformationClass jobObjectInformationClass, IntPtr jobObjectInformationHandle, uint jobObjectInformationLength);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로세스 설정하기 - SetProcess(process, limitUsage)

        /// <summary>
        /// 프로세스 설정하기
        /// </summary>
        /// <param name="process">프로세스</param>
        /// <param name="limitUsage">제한 사용률 (예 : 45 (45%))</param>
        /// <returns>처리 결과</returns>
        public static bool SetProcess(Process process, uint limitUsage)
        {
            IntPtr jobHandle = CreateJobObject(null, null);

            AssignProcessToJobObject(jobHandle, process.Handle);

            JOBOBJECT_CPU_RATE_CONTROL_INFORMATION information = new JOBOBJECT_CPU_RATE_CONTROL_INFORMATION();

            information.ControlFlag = (uint)(ControlFlag.JOB_OBJECT_CPU_RATE_CONTROL_ENABLE | ControlFlag.JOB_OBJECT_CPU_RATE_CONTROL_HARD_CAP);
            information.CPURate      = limitUsage * 100;

            IntPtr informationHandle = Marshal.AllocHGlobal(Marshal.SizeOf(information));

            Marshal.StructureToPtr(information, informationHandle, false);

            bool result = SetInformationJobObject
            (
                jobHandle,
                JobObjectInformationClass.JobObjectCpuRateControlInformation,
                informationHandle,
                (uint)Marshal.SizeOf(information)
            );

            return result;
        }

        #endregion
    }
}